var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { jsx as _jsx } from "react/jsx-runtime";
import * as React from "react";
var useSyncStore = function (subscribe, getSnapshot, getServerSnapshot) {
    var getIsomorphicSnapshot = typeof document !== "undefined"
        ? getSnapshot
        : getServerSnapshot !== null && getServerSnapshot !== void 0 ? getServerSnapshot : getSnapshot;
    var _a = React.useState(getIsomorphicSnapshot), state = _a[0], setState = _a[1];
    var onStoreChange = React.useCallback(function () { return setState(function () { return getIsomorphicSnapshot(); }); }, [getIsomorphicSnapshot]);
    // eslint-disable-next-line react-hooks/exhaustive-deps
    React.useEffect(function () { return subscribe(onStoreChange); }, [onStoreChange]);
    return state;
};
var __STORE_SENTINEL__ = {};
var createStoreContext = function (stateFactory) {
    var StoreContext = React.createContext(null);
    if (process.env.NODE_ENV !== "production")
        StoreContext.displayName = "StoreContext";
    var StoreProvider = function (props) {
        var store = React.useRef(__STORE_SENTINEL__);
        var subscribers = React.useRef(new Set());
        var getState = React.useCallback(function () { return store.current; }, []);
        var setState = React.useCallback(function (setter) {
            var newState = setter(store.current);
            store.current = newState;
            subscribers.current.forEach(function (cb) { return cb(); });
        }, []);
        // Lazy initialization
        if (store.current === __STORE_SENTINEL__)
            store.current = stateFactory(setState, getState);
        var subscribe = React.useCallback(function (onStoreChange) {
            subscribers.current.add(onStoreChange);
            return function () {
                subscribers.current.delete(onStoreChange);
            };
        }, []);
        var context = React.useMemo(function () { return ({ getState: getState, subscribe: subscribe }); }, [getState, subscribe]);
        return (_jsx(StoreContext.Provider, __assign({ value: context }, { children: props.children }), void 0));
    };
    var useStore = function (selector) {
        var storeContext = React.useContext(StoreContext);
        if (!storeContext) {
            throw new Error("[@utilityjs/create-store-context]: You can only use `useStore` in a subtree of `<StoreProvider>`.");
        }
        var getState = storeContext.getState, subscribe = storeContext.subscribe;
        var getSnapshot = React.useCallback(function () { return selector(getState()); }, 
        // eslint-disable-next-line react-hooks/exhaustive-deps
        [selector]);
        return useSyncStore(subscribe, getSnapshot, getSnapshot);
    };
    return { useStore: useStore, StoreProvider: StoreProvider };
};
export default createStoreContext;
